'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var inquirer = _interopDefault(require('inquirer'));
var chalk = _interopDefault(require('chalk'));
var path = _interopDefault(require('path'));
var childProcess = _interopDefault(require('child_process'));
var fs = _interopDefault(require('fs'));
var ncu = _interopDefault(require('npm-check-updates'));
var tape = _interopDefault(require('tape'));
var babelCore = _interopDefault(require('babel-core'));

var exec = childProcess.exec;
// ----------------------------------------------------------------
var runTests = (function (dir, entry) {
  console.log(chalk.yellow("Running project tests..."));
  exec("./node_modules/.bin/nyc --require ./node_modules/babel-register --reporter=lcov --reporter=text ./node_modules/.bin/tape '" + path.resolve(dir) + "/**/*.js' | ./node_modules/.bin/faucet", function (error, stdout, stderr) {
    if (error !== null) {
      console.log("exec error: " + error);
    }
    console.log(stdout);
    console.log(stderr);
  });
})

var exec$1 = childProcess.exec;
// ----------------------------------------------------------------
var rollupFiles = (function (file, output) {
  console.log("Rolling files into bundle...");
  exec$1("BABEL_ENV=build ./node_modules/.bin/rollup --config ./node_modules/ezy/vendor/rollup.config.js " + path.resolve(file) + " -o " + path.resolve(output), function (error, stdout, stderr) {
    if (error !== null) {
      console.log("exec error: " + error);
    }
    console.log("Rolled " + file + " into bundle at " + output + ".");
  });
})
// --config ../vendor/rollup.config.js

var exec$2 = childProcess.exec;
// ----------------------------------------------------------------
var addBabelPreset = (function () {
  fs.stat(path.resolve("./node_modules/babel-preset-es2015"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.yellow("No Babel preset detected..."));
      console.log(chalk.blue("Installing ES2015 Babel preset via NPM..."));
      exec$2("npm i -D babel-preset-es2015", function (error, stdout, stderr) {
        console.log(chalk.green("Added ES2015 Babel preset."));
        if (error !== null) {
          console.log("exec error: " + error);
        }
      });
    } else {
      console.log(chalk.green("Babel preset already exists."));
    }
  });

  fs.stat(path.resolve(".babelrc"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.yellow("No .babelrc file detected..."));
      console.log(chalk.blue("Creating .babelrc file..."));
      var oldFile = fs.createReadStream(path.resolve("./node_modules/ezy/vendor/.babelrc"));
      var newFile = fs.createWriteStream(path.resolve(".babelrc"));
      oldFile.pipe(newFile);
      console.log(chalk.green(".babelrc file created."));
    } else {
      console.log(chalk.green(".babelrc already exists."));
    }
  });
})

// ----------------------------------------------------------------
var addWallabyConfig = (function () {

  fs.stat(path.resolve("wallaby.js"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.yellow("No wallaby.js file detected..."));
      console.log(chalk.blue("Creating wallaby.js file..."));
      var oldFile = fs.createReadStream(path.resolve("./node_modules/ezy/vendor/wallaby.js"));
      var newFile = fs.createWriteStream(path.resolve("wallaby.js"));
      oldFile.pipe(newFile);
      console.log(chalk.green("wallaby.js file created."));
    } else {
      console.log(chalk.green("wallaby.js already exists."));
    }
  });
})

// ----------------------------------------------------------------
var addEzyConfig = (function () {

  fs.stat(path.resolve("ezy.config.json"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.yellow("No ezy.config.json file detected..."));
      console.log(chalk.blue("Creating ezy.config.json file..."));
      var oldFile = fs.createReadStream(path.resolve("./node_modules/ezy/vendor/ezy.config.json"));
      var newFile = fs.createWriteStream(path.resolve("ezy.config.json"));
      oldFile.pipe(newFile);
      console.log(chalk.green("ezy.config.json file created."));
    } else {
      console.log(chalk.green("ezy.config.json already exists."));
    }
  });
})

// ----------------------------------------------------------------
var addESLint = (function () {

  fs.stat(path.resolve(".eslintrc"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.yellow("No .eslintrc file detected..."));
      console.log(chalk.blue("Creating .eslintrc file..."));

      // fs.writeFile(path.resolve(".babelrc"), "{\n  \"presets\": [\"es2015\"]\n}", (err) => {
      //   if (err) throw err;
      //   console.log(chalk.green("Added preset to .babelrc file."));
      // });
      var oldFile = fs.createReadStream(path.resolve("./node_modules/ezy/vendor/.eslintrc"));
      var newFile = fs.createWriteStream(path.resolve(".eslintrc"));
      oldFile.pipe(newFile);
      console.log(chalk.green(".eslintrc file created."));
    } else {
      console.log(chalk.green(".eslintrc already exists."));
    }
  });
})

var exec$6 = childProcess.exec;
// ----------------------------------------------------------------
var viewCoverage = (function () {
  console.log(chalk.yellow("Opening coverage report in the browser..."));
  exec$6("open " + path.resolve("coverage/lcov-report/index.html"), function (error, stdout, stderr) {
    if (error !== null) {
      console.log("exec error: " + error);
    }
  });
})

var exec$7 = childProcess.exec;
// ----------------------------------------------------------------
var runProject = {
  node: function node(file) {
    console.log(chalk.yellow("Running project with Node..."));
    exec$7("node " + path.resolve(file), function (error, stdout, stderr) {
      if (error !== null) {
        console.log("exec error: " + error);
      }
      console.log(stdout);
      console.log(stderr);
    });
  },
  browser: function browser(file) {
    console.log(chalk.yellow("Running project with BrowserSync..."));
    exec$7("./node_modules/.bin/browser-sync start --server " + path.resolve(file) + " --files " + path.resolve(file), function (error, stdout, stderr) {
      if (error !== null) {
        console.log("exec error: " + error);
      }
      console.log(stdout);
      console.log(stderr);
    });
  }
};

// ----------------------------------------------------------------
var updateDependencies = {
  check: function check() {
    console.log(chalk.yellow("Checking for new versions of your dependencies..."));
    ncu.run({
      packageFile: path.resolve("package.json"),
      jsonUpgraded: true,
      upgrade: false
    }).then(function (toUpgrade) {
      if (Object.getOwnPropertyNames(toUpgrade).length < 1) {
        console.log(chalk.green("Everything is up to date. :)"));
      } else {
        console.log(toUpgrade);
      }
    });
  },
  run: function run() {
    console.log(chalk.yellow("Updating your dependencies..."));
    ncu.run({
      packageFile: path.resolve("package.json"),
      jsonUpgraded: true,
      upgrade: true
    }).then(function (toUpgrade) {
      if (Object.getOwnPropertyNames(toUpgrade).length < 1) {
        console.log(chalk.green("Everything is up to date. :)"));
      } else {
        console.log(toUpgrade);
      }
    });
  }

};

// ----------------------------------------------------------------

var questions = [{
  type: "list",
  name: "inputValue",
  message: "What would you like to do?",
  choices: ["1. Run Project", new inquirer.Separator(), "2. Test Project", "3. View Coverage", new inquirer.Separator(), "4. Bundle Project", new inquirer.Separator(), "5. Check Updates", "6. Run Updates", "7. Add Babel Preset", "8. Add Wallaby Config", "9. Add ESLint", "10. Add ezy Config", new inquirer.Separator()]
}];

var taskList = (function (configure) {
  var config = require(configure); // Weird Require.
  var entry, entryDir, outputDir, output, testDir;
  if (config.entry) {
    entry = config.entry;
  } else {
    entry = "src/index.js";
  }
  if (config.entryDir) {
    entryDir = config.entryDir;
  } else {
    entryDir = "src";
  }
  if (config.outputDir) {
    outputDir = config.outputDir;
  } else {
    outputDir = "dist";
  }
  if (config.output) {
    output = config.output;
  } else {
    output = "bundle.js";
  }
  if (config.testDir) {
    testDir = config.testDir;
  } else {
    testDir = "tests";
  }
  // --------------------------------
  inquirer.prompt(questions).then(function (answers) {
    var a = answers.inputValue;

    if (a === "1. Run Project") {
      inquirer.prompt([{
        type: "list",
        name: "runWhere",
        message: "Where would you like to run the project?",
        choices: ["Node", "Browser"]
      }]).then(function (answers) {
        if (answers.runWhere === "Node") {
          runProject.node(outputDir + "/" + output);
        } else if (answers.runWhere === "Browser") {
          runProject.browser(outputDir);
        }
      });
    } else if (a === "2. Test Project") {
      runTests(testDir, entryDir);
    } else if (a === "3. View Coverage") {
      viewCoverage();
    } else if (a === "4. Bundle Project") {
      rollupFiles(entryDir + "/" + entry, outputDir + "/" + output);
    } else if (a === "5. Check Updates") {
      updateDependencies.check();
    } else if (a === "6. Run Updates") {
      updateDependencies.run();
    } else if (a === "7. Add Babel Preset") {
      addBabelPreset();
    } else if (a === "8. Add Wallaby Config") {
      addWallabyConfig();
    } else if (a === "9. Add ESLint") {
      addESLint();
    } else if (a === "10. Add ezy Config") {
      addEzyConfig();
    }
  });
})

// ----------------------------------------------------------------
var bin = function bin() {
  // TODO: Check for ezy.config.json, chck for ezy section in package json,
  //       else suggest config file is made.
  fs.stat(path.resolve("ezy.config.json"), function (err, stats) {
    if (err && err.code !== "ENOENT") console.log(err);
    if (err && err.code === "ENOENT") {
      console.log(chalk.red("No ezy.config.json file has been found."));
      inquirer.prompt([{
        type: "confirm",
        name: "install",
        message: "Create an ezy.config.json file now?",
        default: true
      }]).then(function (answers) {
        if (answers.install === true) {
          addEzyConfig();
        } else {
          console.log("Please create a config file to use ezy.");
        }
      });
    } else {
      taskList(path.resolve("ezy.config.json"));
    }
  });
};

var test = tape;
var color = chalk;
var babel = babelCore;

exports.bin = bin;
exports.test = test;
exports.color = color;
exports.babel = babel;